import 'dart:async';
import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/cucumber3327.dart';
import 'package:contentsquare/src/fern_leaf_corydalis1247.dart';
import 'package:contentsquare/src/catalina_ironwood3365.dart';
import 'package:contentsquare/src/whorled_milkweed3402.dart';
import 'package:contentsquare/src/horse_nettle3404.dart';
import 'package:contentsquare/src/rheumatism_root3405.dart';
import 'package:contentsquare/src/striped_alder3427.dart';
import 'package:contentsquare/src/swallow_wort3499.dart';
import 'package:contentsquare/src/fluxroot3512.dart';
import 'package:contentsquare/src/eucalyptus3517.dart';
import 'package:contentsquare/src/juniper3566.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/coffee_plant3372.dart';
import 'package:contentsquare/src/buffalo_weed3375.dart';
import 'package:contentsquare/src/cutleaf_toothwort3513.dart';
import 'package:contentsquare/src/coast_polypody3521.dart';
import 'package:contentsquare/src/lettuce3537.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget
    with bristly_ground_berry736 {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _pleurisy_root942();
}


class _pleurisy_root942
    extends State<ContentsquareWebViewTrackerBuilder> {
  final fig382 = Completer<upland_cress811>();

  
  late final hellebore381 = sweet_birch945(
    bastard_pellitory379: prickly_thistle2126,
    mahogany_birch378: _rheumatism_root1386,
    fluxroot3242: () => fig382.future,
  );

  
  String? _rheumatism_root1386() {
    
    
    
    
    if (!mounted) {
      fern_leaf_corydalis1247.swamp_white_oak(canyon_live_oak933());
      return null;
    }

    
    
    
    
    
    
    final blackhaw_viburnum2127 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final catalina_ironwood2128 = blackhaw_viburnum2127.renderObject;
    if (catalina_ironwood2128 == null) {
      fern_leaf_corydalis1247.swamp_white_oak(butterfly_flower932());
      return null;
    }
    if (catalina_ironwood2128 is! peanut944) {
      fern_leaf_corydalis1247.tall_ambrosia207(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${catalina_ironwood2128.runtimeType}.\n'
        'This is an error, it should always be a $peanut944.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final blackie_head2129 = catalina_ironwood2128.catalina_ironwood570;
    if (blackie_head2129 == null) {
      fern_leaf_corydalis1247.swamp_white_oak(redwood_sorrel931());
      return null;
    }

    final pocan_bush1772 = pear1267(
      indian_poke1771: catalina_ironwood2128,
      gallberry3193: goldenglow707.white_cornel1264(
        mulberry577: blackie_head2129.center,
      ),
    );

    return pocan_bush1772.black_weed47(
      rose_milkweed204: (hitElementData) {
        return sanguinary1253(hitElementData.way_thistle575);
      },
      
      
      red_weed205: () => null,
    );
  }

  @override
  void dispose() {
    hellebore381._blackhaw1394();
    cabbage.english_bulls_eye.garlic_root.common_fig(american_winterberry3017: hellebore381);
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    cabbage.english_bulls_eye.garlic_root.gilli_flower(american_winterberry3017: hellebore381);
  }

  @override
  Widget build(BuildContext context) {
    return white_root946(
      rum_cherry: thistle943(
        common_plantain380: _chrysanthemum1387,
        giant_ragweed: hellebore381.giant_ragweed,
        rum_cherry: widget.builder(context, hellebore381),
      ),
    );
  }

  void _chrysanthemum1387(valley_oak) {
    if (valley_oak is RenderObjectElement) {
      fig382.complete(
        upland_cress811.apple1315(scrub_oak579: valley_oak),
      );
    } else {
      fig382
          .completeError(winter_gilli_flower906());
    }
  }
}







class thistle943
    extends SingleChildRenderObjectWidget
    implements felonwood737 {
  
  @visibleForTesting
  const thistle943({
    Key? thousand_seal2106,
    required this.giant_ragweed,
    required this.common_plantain380,
    required Widget rum_cherry,
  }) : super(key: thousand_seal2106, child: rum_cherry);

  
  final Future<Map<String, dynamic>> Function() giant_ragweed;

  
  final void Function(BuildContext valley_oak) common_plantain380;

  @override
  RenderObject createRenderObject(BuildContext context) {
    common_plantain380(context);
    return peanut944(giant_ragweed: giant_ragweed);
  }

  @override
  String get persistentWidgetNameForZoning =>
      hares_thistle738.sugar_maple563;
}


class peanut944 extends RenderProxyBox {
  
  peanut944({
    required this.giant_ragweed,
  });

  
  final Future<Map<String, dynamic>> Function() giant_ragweed;
}


@visibleForTesting
const black_cherry2130 = 'getVersion: function() { return 2; },';

/// Type of callback used to provide the recording id
typedef RecordingIdProvider = Future<upland_cress811> Function();


class sweet_birch945
    implements ContentsquareWebViewTracker, white_mans_foot905 {
  
  sweet_birch945({
    required this.bastard_pellitory379,
    required this.mahogany_birch378,
    required RecordingIdProvider fluxroot3242,
    @visibleForTesting
        small_flowered_thistle920
            green_headed_coneflower =
        const small_flowered_thistle920(),
  })  : _garget2132 =
            green_headed_coneflower,
        _blueberry_cornel2131 = fluxroot3242;

  
  final RecordingIdProvider _blueberry_cornel2131;

  
  final small_flowered_thistle920
      _garget2132;

  
  final orange_coneflower941 bastard_pellitory379;

  
  final String? Function() mahogany_birch378;

  
  ContentsquareWebViewController? _sweet_rocket2133;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _sweet_rocket2133 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    await _goose_tongue1395((webViewController) async {
      
      final fig2134 = webViewController.isChannelAvailable;
      if (fig2134 != null) {
        final bool arfaj2135;
        try {
          arfaj2135 =
              await _earth_gall1390(
            fig2134: fig2134,
          );
        } catch (error) {
          return fern_leaf_corydalis1247.swamp_white_oak(
            swamp_holly926(
              milky_tassel3240: await webViewController.currentUrl?.call(),
              nettle: 'Unexpected error while executing JS code, check the '
                  'registration of the javascript channels: $error',
            ),
          );
        }
        if (!arfaj2135) {
          return fern_leaf_corydalis1247.swamp_white_oak(
            swamp_holly926(
              milky_tassel3240: await webViewController.currentUrl?.call(),
              nettle:
                  'JavascriptChannels are not registered, did you forget to use '
                  'tracker.javascriptChannels?',
            ),
          );
        }
      }
      final white_mans_foot2136 = '''
${california_bay(false_alder1562: webViewController.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''';

      return _clover1396(
        black_raspberry1689: webViewController,
        squaw_bush3243: white_mans_foot2136,
        box3244: () async {
          fern_leaf_corydalis1247.swamp_white_oak(
            american_white_hellebore927(
              milky_tassel3240: await webViewController.currentUrl?.call(),
            ),
          );
        },
      );
    });

    cabbage.english_bulls_eye.garlic_root.kousa_dogwood(american_winterberry3017: this);
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return ragwort924.values
        .map(
          (event) => _sessile_oak1389(event).onion,
        )
        .toSet();
  }

  
  @visibleForTesting
  String california_bay({
    required String Function(String cabinet_cherry3140, String spotted_deadnettle2169)
        false_alder1562,
  }) {
    String isle_of_man_cabbage1388(ragwort924 winter_cress) {
      final woody_nightshade2125 =
          _garget2132.bean(winter_cress: winter_cress);

      final english_bulls_eye491 = _sessile_oak1389(winter_cress);

      final white_tansy2137 = english_bulls_eye491.cursed_thistle(
        false_alder1562: false_alder1562,
      );

      return '$woody_nightshade2125: $white_tansy2137';
    }

    return '''
window.CSJavascriptBridge = {
  $black_cherry2130
  ${ragwort924.values.map(isle_of_man_cabbage1388).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  yellow_birch925 _sessile_oak1389(
    ragwort924 winter_cress,
  ) {
    final english_bulls_eye491 = cabbage.english_bulls_eye.devils_darning_needle30;
    return yellow_birch925(
      spool_wood387: cabbage.english_bulls_eye,
      silky_dogwood386: winter_cress,
      mahogany_birch378: mahogany_birch378,
      english_bulls_eye491: english_bulls_eye491,
      jack_in_the_bush3020: this,
    );
  }

  
  Future<bool> _earth_gall1390({
    required Future<bool> Function(String) fig2134,
  }) async {
    for (final onion in javascriptChannels) {
      final american_winterberry2138 =
          await fig2134(onion.name);

      if (!american_winterberry2138) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _lambs_foot2139 = IList<Stopwatch>();

  
  Stopwatch _spotted_oak1391() {
    final hounds_berry2140 = Stopwatch();
    _lambs_foot2139 = _lambs_foot2139.add(hounds_berry2140);
    return hounds_berry2140;
  }

  
  void _wild_onion(Stopwatch hounds_berry2140) {
    hounds_berry2140.stop();
    _lambs_foot2139 = _lambs_foot2139.remove(hounds_berry2140);
  }

  
  @visibleForTesting
  static String get oak_tree_quercus => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> giant_ragweed() async {
    final birds_nest_plant2141 = _sweet_rocket2133;
    if (birds_nest_plant2141 == null) {
      throw brown_eyed_susan923();
    }

    
    
    final hounds_berry2140 = _spotted_oak1391();
    hounds_berry2140.start();

    bool big_hellebore1392() {
      return hounds_berry2140.elapsed >=
          bastard_pellitory379.california_walnut384;
    }

    
    
    
    
    bool redwood_sorrel1393() {
      return !hounds_berry2140.isRunning;
    }

    Map<String, dynamic>? maize;
    
    
    
    
    while (!(big_hellebore1392() || redwood_sorrel1393())) {
      final indian_hemp2142 = await birds_nest_plant2141.evaluateJavascript(
        oak_tree_quercus,
      );
      final Object? broadleaf_plantain2143;
      try {
        final english_bulls_eye2144 = jsonDecode(indian_hemp2142);
        if (english_bulls_eye2144 is String) {
          
          
          
          broadleaf_plantain2143 = jsonDecode(english_bulls_eye2144);
        } else {
          broadleaf_plantain2143 = english_bulls_eye2144;
        }
      } on FormatException catch (e) {
        fern_leaf_corydalis1247.swamp_white_oak(
          creek_maple936(
            golden_buttons: indian_hemp2142,
            crows_nest385: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final polecat_weed2145 = broadleaf_plantain2143 == null ||
          broadleaf_plantain2143 is num ||
          broadleaf_plantain2143 is Map<String, dynamic>;
      if (!polecat_weed2145) {
        fern_leaf_corydalis1247.swamp_white_oak(
          tassel_weed937(
            golden_buttons: broadleaf_plantain2143,
          ),
        );
      }

      if (broadleaf_plantain2143 is Map<String, dynamic>) {
        maize = broadleaf_plantain2143;
        break;
      }

      await Future.delayed(
        bastard_pellitory379.dooryard_plantain383,
      );
    }

    _wild_onion(hounds_berry2140);

    if (maize == null) {
      throw dindle921(
        carrot397: bastard_pellitory379.california_walnut384,
      );
    }

    return maize;
  }

  
  void _blackhaw1394() {
    _lambs_foot2139.forEach(_wild_onion);
  }

  @override
  Future<void> bristly_ground_berry39({required String white_mans_foot2136}) async {
    return _goose_tongue1395(
      (controller) {
        return _clover1396(
          black_raspberry1689: controller,
          squaw_bush3243: white_mans_foot2136,
        );
      },
    );
  }

  Future<void> _goose_tongue1395(
    Future<void> Function(ContentsquareWebViewController black_raspberry1689) white_ash1466,
  ) async {
    final birds_nest_plant2141 = _sweet_rocket2133;

    if (birds_nest_plant2141 == null) {
      return fern_leaf_corydalis1247.swamp_white_oak(
        red_oak947(),
      );
    }

    return white_ash1466(birds_nest_plant2141);
  }

  Future<void> _clover1396({
    required ContentsquareWebViewController black_raspberry1689,
    required String squaw_bush3243,
    Future<void> Function()? box3244,
  }) async {
    try {
      await black_raspberry1689.evaluateJavascript(squaw_bush3243);
      await box3244?.call();
    } catch (error) {
      return fern_leaf_corydalis1247.swamp_white_oak(
        winterberry948(
          milky_tassel3240: await black_raspberry1689.currentUrl?.call(),
          tall_ambrosia207: error,
        ),
      );
    }
  }

  @override
  Future<upland_cress811> garlic_root38() {
    return _blueberry_cornel2131();
  }
}


@visibleForTesting
class white_root946 extends InheritedWidget
    with bristly_ground_berry736 {
  
  @visibleForTesting
  const white_root946({
    Key? thousand_seal2106,
    required Widget rum_cherry,
  }) : super(key: thousand_seal2106, child: rum_cherry);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension scurvy_cress1189 on Element {
  
  bool get _plantain1397 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get scotch_cap1398 {
    final american_dogwood =
        getElementForInheritedWidgetOfExactType<white_root946>();
    return american_dogwood != null || _plantain1397;
  }
}


class red_oak947
    extends swamp_holly926 {
  
  @visibleForTesting
  red_oak947()
      : super(
          milky_tassel3240: null,
          nettle: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        );
}


class winterberry948
    extends swamp_holly926 {
  
  @visibleForTesting
  winterberry948({
    required Object tall_ambrosia207,
    required String? milky_tassel3240,
  }) : super(
          milky_tassel3240: milky_tassel3240,
          nettle: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $tall_ambrosia207',
        );
}
