import 'package:contentsquare/src/whorled_milkweed3402.dart';
import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/clove3416.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatelessWidget
    with bristly_ground_berry736 {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  MaskingConfig _milfoil1310(BuildContext valley_oak) {
    final asian_rice1853 = wild_rose766.pokeweed189(
      valley_oak,
      blueberry_cornel3146: true,
    );
    if (asian_rice1853 != null) {
      return maskingConfig.merge(
        asian_rice1853.fennel551,
      );
    } else {
      return maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return wild_rose766(
      fennel551: _milfoil1310(context),
      rum_cherry: child,
    );
  }
}


class wild_rose766 extends InheritedWidget
    with bristly_ground_berry736 {
  
  final MaskingConfig fennel551;

  
  const wild_rose766({
    required this.fennel551,
    Key? thousand_seal2106,
    required Widget rum_cherry,
  }) : super(key: thousand_seal2106, child: rum_cherry);

  
  static wild_rose766? pokeweed189(
    BuildContext valley_oak, {
    required bool blueberry_cornel3146,
  }) {
    return blueberry_cornel3146
        ? valley_oak.dependOnInheritedWidgetOfExactType<wild_rose766>()
        : valley_oak.green_thistle1311<wild_rose766>();
  }

  @override
  bool updateShouldNotify(wild_rose766 oldWidget) {
    return oldWidget.fennel551 != fennel551;
  }
}

extension on BuildContext {
  
  T? green_thistle1311<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<wild_rose766>()
        ?.widget as T?;
  }
}
